/*******************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL All rights reserved. * Please look
 * at license.txt in info directory for more license detail. *
 ******************************************************************************/
package org.exoplatform.services.resources.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.commons.logging.Log;
import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.commons.utils.MapResourceBundle;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.resources.LocaleConfig;
import org.exoplatform.services.resources.LocaleConfigService;
import org.exoplatform.services.resources.Query;
import org.exoplatform.services.resources.ResourceBundleData;
import org.exoplatform.services.resources.ResourceBundleService;
import org.hibernate.Session;

public class ResourceBundleServiceImpl implements ResourceBundleService {
	private HibernateService hService_;
	private Log log_;
	private ExoCache cache_;
	private List classpathResources_ ;
	private LocaleConfigService localeService_;
  private String[]  portalResourceBundleNames_ ;

	public ResourceBundleServiceImpl(HibernateService service,
			                             LocaleConfigService localeService, LogService lservice,
			                             CacheService cService, InitParams params) throws Exception {
	  log_ = lservice.getLog("org.exoplatform.services.resources");
	  cache_ = cService.getCacheInstance(getClass().getName());
	  
	  localeService_ = localeService;
	  hService_ = service ;
	  classpathResources_ = params.getValuesParam("classpath.resources").getValues();
    
    List  prnames = params.getValuesParam("portal.resource.names").getValues();
    portalResourceBundleNames_ = new String[prnames.size()] ;
    for(int i = 0; i < prnames.size(); i++) portalResourceBundleNames_[i] = (String)prnames.get(i) ; 
    
    PageList pl  = findResourceDescriptions(new  Query(null, null)) ;
    if(pl.getAvailable() == 0) {
      List<String> initResources = params.getValuesParam("init.resources").getValues();
      for(String resource : initResources) {
        initResources(resource, Thread.currentThread().getContextClassLoader()) ;
      }
    }
    
	}

  public String[] getPortalResourceBundleNames() { return portalResourceBundleNames_ ; }
  
	public ResourceBundle getResourceBundle(String name, Locale locale) {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		return getResourceBundle(name, locale, cl);
	}

	public ResourceBundle getResourceBundle(String name, Locale locale,	ClassLoader cl) {
	  if (isClasspathResource(name))	return ResourceBundle.getBundle(name, locale, cl);
	  String id = name + "_" + locale.getLanguage();
	  try {
	    Object obj = cache_.get(id);
	    if (obj != null) {
	      return (ResourceBundle) obj;
	    }
	  } catch (Exception ex) {}
	  
	  try {
	    ResourceBundle res = null;
	    String rootId = name + "_" + localeService_.getDefaultLocaleConfig().getLanguage();
	    ResourceBundle parent = getResourceBundleFromDb(rootId, null, locale);
	    if (parent != null) {
	      res = getResourceBundleFromDb(id, parent, locale);
	      if (res == null) res = parent;
	      cache_.put(id, res);
	      return res;
	    }
	  } catch (Exception ex) {
	    log_.error("Error: " + id, ex);
	  }
	  return null;
	}

	public ResourceBundle getResourceBundle(String[] name, Locale locale) {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		return getResourceBundle(name, locale, cl);
	}

	public ResourceBundle getResourceBundle(String[] name, Locale locale, ClassLoader cl) {
		StringBuilder idBuf = new StringBuilder("merge:");
		for (String n : name) idBuf.append(n).append("_");
		idBuf.append(locale.getLanguage());
		String id = idBuf.toString();
		try {
			ResourceBundle 	res = (ResourceBundle) cache_.get(id);
			if (res != null) return res;
			MapResourceBundle outputBundled = new MapResourceBundle(locale);
			for (int i = 0; i < name.length; i++) {
				ResourceBundle temp = getResourceBundle(name[i], locale, cl);
				if (temp != null)	outputBundled.merge(temp);
			}
			outputBundled.resolveDependencies();
			cache_.put(id, outputBundled);
			return outputBundled;
		} catch (Exception ex) {
			log_.error("Cannot load and merge the bundle: " + id, ex);
		}
		return null;
	}

	public ResourceBundleData getResourceBundleData(String name) throws Exception {
		return (ResourceBundleDataImpl) hService_.findOne(ResourceBundleDataImpl.class, name);
	}

	public ResourceBundleData removeResourceBundleData(String id) throws Exception {
	  ResourceBundleDataImpl data =  (ResourceBundleDataImpl) hService_.remove(ResourceBundleDataImpl.class, id);
	  cache_.remove(data.getId());
	  return data;
	}
	
	public PageList findResourceDescriptions(Query q) throws Exception {
		String name = q.getName();
		if (name == null || name.length() == 0)	name = "%";
		ObjectQuery oq = new ObjectQuery(ResourceBundleDescriptionImpl.class);
		oq.addLIKE("name", name);
		oq.addLIKE("language", q.getLanguage());
		oq.setDescOrderBy("name");
		return new DBObjectPageList(hService_, oq);
	}

	public void saveResourceBundle(ResourceBundleData data) throws Exception {
		hService_.save(data);
		cache_.remove(data.getId());
	}

	private ResourceBundle getResourceBundleFromDb(String id,	ResourceBundle parent, 
                                                 Locale locale) throws Exception {
		Session session = hService_.openSession();
		ResourceBundleData data = 
      (ResourceBundleDataImpl) session.get(ResourceBundleDataImpl.class, id);
		if (data != null) {
			InputStream is = new ByteArrayInputStream(data.getData().getBytes());
			ResourceBundle res = new ExoResourceBundle(is, parent);
      MapResourceBundle mres = new MapResourceBundle(res, locale) ;
			return mres;
		}
		return null;
	}

	private void initResources(String baseName, ClassLoader cl) {
		String name = baseName.replace('.', '/');
		String fileName = null;
		try {
			Collection localeConfigs = localeService_.getLocalConfigs();
			String defaultLang = localeService_.getDefaultLocaleConfig().getLanguage();
			for (Iterator iter = localeConfigs.iterator(); iter.hasNext();) {
				LocaleConfig localeConfig = (LocaleConfig) iter.next();
				String language = localeConfig.getLanguage();
				if (defaultLang.equals(language)) {
					fileName = name + ".properties";
				} else {
					fileName = name + "_" + language + ".properties";
				}
				URL url = cl.getResource(fileName);
				if (url != null) {
					InputStream is = url.openStream();
					byte buf[] = IOUtil.getStreamContentAsBytes(is);
					ResourceBundleDataImpl data = new ResourceBundleDataImpl();
					data.setName(baseName);
					data.setLanguage(language);
					data.setData(new String(buf, "UTF-8"));
					saveResourceBundle(data);
				}
			}
		} catch (Exception ex) {
			log_.error("Error while reading the file: " + fileName, ex);
		}
	}

	public ResourceBundleData createResourceBundleDataInstance() {
		return new ResourceBundleDataImpl();
	}

	private boolean isClasspathResource(String name) {
		if (classpathResources_ == null)	return false;
		for (int i = 0; i < classpathResources_.size(); i++) {
			String pack = (String) classpathResources_.get(i);
			if (name.startsWith(pack)) return true;
		}
		return false;
  }
}
